require 'resolv'
require 'ipaddress'

class VpnService < ActiveRecord::Base
  attr_accessible :member_id, :login, :expired_at, :password, :ipv4

  belongs_to :member

  before_validation :ensure_password_is_generated
  before_validation :ensure_login_is_generated
  before_validation :ensure_ipv4_is_selected

  validates :member, :expired_at, :password, :presence => true
  validates :password, :length => { :minimum => 16 }

  validates :login, :presence => true
  validates :login, :uniqueness => true,
  :format => { :with => /^vpn\d+$/ }, on: :create
  validates :ipv4, :presence => true
  validates :ipv4, :uniqueness => true,
  :format => { :with => Resolv::IPv4::Regex }, on: :create

  validate :expiration_date_cannot_be_in_the_past
  validate :ipv4_must_be_under_the_network_range
  validate :ipv4_should_not_already_exist_in_the_network_range, on: :create
  validate :ipv4_should_not_be_the_first_or_last_in_the_network_range

  protected
  def ensure_password_is_generated
      self.password = SecureRandom.base64(16) unless password.present?
  end

  def ensure_login_is_generated
    if !self.login.nil?
      return true
    end
    self.login = ""
    i = 1
    while self.login.empty? or VpnService.where(:login => self.login).exists?
      self.login = 'vpn'+i.to_s
      i += 1
    end
  end

  def expiration_date_cannot_be_in_the_past
    if !expired_at.blank? and expired_at <= Date.today
      errors.add(:expired_at, "can't be in the past")
    end
  end

  def ensure_ipv4_is_selected
    if !self.ipv4.nil?
      return true
    end
    network_vpn = Network.find_by_name("vpn")
    netmask_vpn = IPAddress::IPv4.new(network_vpn.netmask_ipv4)
    netmask_vpn.each_host do |addr|
      if VpnService.find_by_ipv4(addr.address).nil? and addr.address != netmask_vpn.first.address and addr.address != netmask_vpn.last.address
        self.ipv4 = addr.to_s
        break
      end
    end
  end

  def ipv4_must_be_under_the_network_range
    return if self.ipv4.nil?
    return if self.ipv4.to_s.match(Resolv::IPv4::Regex).nil?
    network_vpn_str = Network.find_by_name("vpn")
    network_vpn_obj = IPAddress::IPv4.new(network_vpn_str.netmask_ipv4)
    current_ipv4 = IPAddress::IPv4.new(self.ipv4)
    if !network_vpn_obj.include? current_ipv4
      errors.add(:out_of_range, "IP out of range")
    end
  end

  def ipv4_should_not_be_the_first_or_last_in_the_network_range
    return if self.ipv4.nil?
    return if self.ipv4.to_s.match(Resolv::IPv4::Regex).nil?
    network_vpn_str = Network.find_by_name("vpn")
    network_vpn_obj = IPAddress::IPv4.new(network_vpn_str.netmask_ipv4)
    current_ipv4 = IPAddress::IPv4.new(self.ipv4)
    if network_vpn_obj.last.to_s == current_ipv4.to_s
      errors.add(:out_of_range, "IP last of range")
    end
    if network_vpn_obj.first.to_s == current_ipv4.to_s
      errors.add(:out_of_range, "IP first of range")
    end
  end

  def ipv4_should_not_already_exist_in_the_network_range
    return if self.ipv4.nil?
    return if self.ipv4.to_s.match(Resolv::IPv4::Regex).nil?
    if !VpnService.find_by_ipv4(self.ipv4).nil?
      errors.add(:ip_already_used, "IP already used")
    end
  end

end
