require 'spec_helper'

describe MembersController do
  describe "GET 'index'" do
    before(:each) do
      @member = FactoryGirl.create(:member)
      @fabien = FactoryGirl.create(:fabien)
    end

    context 'when not logged in' do
      it 'should redirect me to the login page' do
        get :index
        response.should redirect_to(new_member_session_path)
      end
    end

    context 'when logged in as a simple member' do
      before(:each) do
        sign_in @member
      end

      it 'should not show other members' do
        get :index
        assigns(:members).should == [@member]
      end
    end

    context 'when logged in as a board member' do
      before(:each) do
        @board_member = FactoryGirl.create(:board_member)
        sign_in @board_member
      end

      it "should be successful" do
        get :index
        response.should be_success
      end
    end
  end
  
  describe "GET 'show'" do
    before(:each) do
      @member = FactoryGirl.create(:member)
    end

    context 'when not logged in' do
      it 'should redirect me to the login page' do
        get :show, :id => @member.id
        response.should redirect_to(new_member_session_path)
      end
    end

    context 'when logged in as a simple member' do
      before(:each) do
        @member_fabien = FactoryGirl.create(:fabien)
        sign_in @member
      end

      it 'should deny me access' do
        expect {
          get :show, :id => @member_fabien.id
        }.to raise_error(CanCan::AccessDenied)
      end
      
      it "should be successful" do
        get :show, :id => @member.id
        response.should be_success
      end
      
    end

    context 'when logged in as a board member' do
      before(:each) do
        @board_member = FactoryGirl.create(:board_member)
        sign_in @board_member
      end

      it "should be successful" do
        get :show, :id => @member.id
        response.should be_success
      end

      it "should find the right member" do
        get :show, :id => @member.id
        assigns(:member).should == @member
      end

      context "when a member has paid this year's subscription" do
        it 'should not prepare a new subscription' do
          get :show, :id => @member.id
          assigns(:new_subscription).should be_nil
        end
      end

      context "when a member has not paid this year's subscription" do
        before(:each) do
          @not_up_to_date = FactoryGirl.create(:fabien, :subscriptions_paid => false)
        end

        it 'should prepare a new subscription for this year' do
          get :show, :id => @not_up_to_date.id
          assigns(:new_subscription).should be_a(Subscription)
          assigns(:new_subscription).year.should == Time.now.year
          assigns(:new_subscription).member.should == @not_up_to_date
        end
      end
    end
  end

  describe "GET 'edit'" do
    before(:each) do
      @member = FactoryGirl.create(:member)
    end

    context 'when not logged in' do
      it 'should redirect me to the login page' do
        get :edit, :id => @member.id
        response.should redirect_to(new_member_session_path)
      end
    end

    context 'when logged in as a simple member' do
      before(:each) do
        @member_fabien = FactoryGirl.create(:fabien)
        sign_in @member
      end

      it 'should deny access to other members' do
        expect {
          get :edit, :id => @member.id
        }.to raise_error(CanCan::AccessDenied)
      end

      it 'should deny access to my own data' do
        expect {
          get :edit, :id => @member_fabien.id
        }.to raise_error(CanCan::AccessDenied)
      end
    end

    context 'when logged in as a board member' do
      before(:each) do
        @board_member = FactoryGirl.create(:board_member)
        sign_in @board_member
      end

      it "should be successful" do
        get :edit, :id => @member.id
        response.should be_success
      end

      it "should find the right member" do
        get :edit, :id => @member.id
        assigns(:member).should == @member
      end
    end
  end

  describe "PUT 'update'" do
    before(:each) do
      @member = FactoryGirl.create(:member)
    end

    context 'when not logged in' do
      it 'should redirect me to the login page' do
        post :update, :id => @member.id
        response.should redirect_to(new_member_session_path)
      end
    end

    context 'when logged in as a simple member' do
      before(:each) do
        @member_fabien = FactoryGirl.create(:fabien)
        sign_in @member
      end

      it 'should deny access to other members' do
        expect {
          post :update, :id => @member.id
        }.to raise_error(CanCan::AccessDenied)
      end

      it 'should deny access to my own data' do
        expect {
          post :update, :id => @member_fabien.id
        }.to raise_error(CanCan::AccessDenied)
      end
    end

    context 'when logged in as a board member' do
      before(:each) do
        @board_member = FactoryGirl.create(:board_member)
        sign_in @board_member
      end

      it "should be successful" do
        post :update, :id => @member.id,
             :member => { :name => @member.name, :email => @member.email,
                          :address => @member.address  }
        response.should redirect_to(member_path(@member))
      end

      it "should find the right member" do
        post :update, :id => @member.id
        assigns(:member).should == @member
      end

      it "should save the right data" do
        post :update, :id => @member.id,
             :member => { :name => 'Test', :email => @member.email,
                          :address => @member.address  }
        Member.find(@member.id).name.should == 'Test'
      end
    end
  end

  describe "GET 'new'" do
    context 'when not logged in' do
      it 'should redirect me to the login page' do
        get :new
        response.should redirect_to(new_member_session_path)
      end
    end

    context 'when logged in as a simple member' do
      before(:each) do
        @member = FactoryGirl.create(:member)
        sign_in @member
      end

      it 'should deny me access' do
        expect {
          get :new
        }.to raise_error(CanCan::AccessDenied)
      end
    end

    context 'when logged in as a board member' do
      before(:each) do
        @board_member = FactoryGirl.create(:board_member)
        sign_in @board_member
      end

      it 'should be successful' do
        get :new
        response.should be_success
      end
      it 'should assign a new member' do
        get :new
        assigns(:member).should be_a(Member)
        assigns(:member).id.should be_nil
      end
    end
  end
end
