require 'spec_helper'

describe Member do
  it "should create a new instance given a valid attribute" do
    attrs = FactoryGirl.build(:member)
  end

  it "should require a name" do
    no_email_member = FactoryGirl.build(:member, :name => '')
    no_email_member.should_not be_valid
  end

  it "should require an email address" do
    no_email_member = FactoryGirl.build(:member, :email => '')
    no_email_member.should_not be_valid
  end

  it "should accept valid email addresses" do
    addresses = %w[member@foo.com THE_USER@foo.bar.org first.last@foo.jp]
    addresses.each do |address|
      valid_email_member = FactoryGirl.build(:member, :email => address)
      valid_email_member.should be_valid
    end
  end

  it "should reject invalid email addresses" do
    addresses = %w[member@foo,com member_at_foo.org example.member@foo.]
    addresses.each do |address|
      invalid_email_member = FactoryGirl.build(:member, :email => address)
      invalid_email_member.should_not be_valid
    end
  end

  it "should reject duplicate email addresses" do
    FactoryGirl.create(:member)
    member_with_duplicate_email = FactoryGirl.build(:member)
    member_with_duplicate_email.should_not be_valid
  end

  it "should reject email addresses identical up to case" do
    FactoryGirl.create(:member, :email => 'TEST@EXAMPLE.ORG')
    member_with_duplicate_email = FactoryGirl.build(:member, :email => 'test@example.org')
    member_with_duplicate_email.should_not be_valid
  end

  describe "passwords" do
    before(:each) do
      @member = FactoryGirl.build(:member)
    end

    it "should have a password attribute" do
      @member.should respond_to(:password)
    end

    it "should have a password confirmation attribute" do
      @member.should respond_to(:password_confirmation)
    end
  end

  describe "password validations" do
    it "should require a password" do
      FactoryGirl.build(:member, :password => '', :password_confirmation => '').
        should_not be_valid
    end

    it "should require a matching password confirmation" do
      FactoryGirl.build(:member, :password_confirmation => 'invalid').
        should_not be_valid
    end

    it "should reject short passwords" do
      short = "a" * 5
      FactoryGirl.build(:member, :password => short, :password_confirmation => short).
        should_not be_valid
    end
  end

  describe "password encryption" do
    before(:each) do
      @member = FactoryGirl.create(:member)
    end

    it "should have an encrypted password attribute" do
      @member.should respond_to(:encrypted_password)
    end

    it "should set the encrypted password attribute" do
      @member.encrypted_password.should_not be_blank
    end
  end

  it "should require a postal address" do
    no_address_member = FactoryGirl.build(:member, :address => '')
    no_address_member.should_not be_valid
  end

  describe "#admin?" do
    context 'when given a member not in the board of directors' do
      let(:member) { FactoryGirl.build(:member) }
      it { member.should_not be_an_admin }
    end
    context 'when given a member in the board of directors' do
      let(:member) { FactoryGirl.build(:member, :admin => true) }
      it { member.should be_an_admin }
    end
  end

  it 'should allow several subscriptions' do
    member = FactoryGirl.create(:member, :joined_on => Time.utc(2010))
    member.subscriptions.count.should == Time.now.year - member.joined_on.year + 1
  end

  it 'should have subscriptions ordered by year' do
    member = FactoryGirl.create(:member, :subscriptions_paid => false)
    FactoryGirl.create(:subscription, :member => member, :year => 2012, :amount => '15,00')
    FactoryGirl.create(:subscription, :member => member, :year => 2011, :amount => '15,00')
    member.subscriptions.last.year.should == 2012
  end

  describe '#last_subscription_paid?' do
    context 'given a member who has paid this year subscription' do
      before(:each) do
        @member = FactoryGirl.create(:member)
      end
      it { @member.last_subscription_paid?.should be_true }
    end
    context 'given a member who has paid 364 days ago his subscription' do
      before(:each) do
        @member = FactoryGirl.create(:member, :subscriptions_paid => false)
        @member.subscriptions.create(:paid_on => Time.now - 364 * 24*3600, :amount => '15,00')
      end
      it { @member.last_subscription_paid?.should be_true }
    end
    context 'given a member who has paid last year subscription' do
      before(:each) do
        @member = FactoryGirl.create(:member, :subscriptions_paid => false)
        @member.subscriptions.create(:paid_on => Time.utc(Time.now.year - 1), :amount => '15,00')
      end
      it { @member.last_subscription_paid?.should be_false }
    end
    context 'given a member who has never paid any subscriptions' do
      before(:each) do
        @member = FactoryGirl.create(:member, :subscriptions_paid => false)
      end
      it { @member.last_subscription_paid?.should be_false }
    end
  end

  describe '#last_subscription' do
    context 'given a member who has never paid any subscriptions' do
      before(:each) do
        @member = FactoryGirl.create(:member, :subscriptions_paid => false)
      end
      it { @member.last_subscription.should be_nil }
    end
    context 'given a member who has paid a subscription' do
      before(:each) do
        @member = FactoryGirl.create(:member, :subscriptions_paid => false)
        @subscription = @member.subscriptions.create(:paid_on => Time.utc(Time.now.year - 1), :amount => '15,00')
      end
      it { @member.last_subscription.should == @subscription }
    end
    context 'given a member who has paid an unpaid subscription' do
      before(:each) do
        @member = FactoryGirl.create(:member, :subscriptions_paid => false)
        @paid_sub = @member.subscriptions.create(:paid_on => Time.utc(2011), :amount => '15,00')
        @new_sub = @member.subscriptions.build(:paid_on => Time.utc(2012), :amount => '15,00')
      end
      it { @member.last_subscription.should == @paid_sub }
    end
  end
end
